require 'Core/DYBaseObj'


module DYAutomate

  class DYAutomateConfigDetail < DYBaseObj
    attr_accessor :templates_git_url
    attr_accessor :codeSnippsets_git_url
    attr_accessor :private_repo_name
    attr_accessor :env_str
  end

  class DYAutomateConfig < DYBaseObj
    @public
    #cache相关的路径
    @@cachePath = File.join(Dir.home(),'Documents','.DYAutomate')
    @@configFilePath = File.join(@@cachePath, 'DYConfigFile.rb')
    @@configGitClonePath = File.join(@@cachePath, 'Git')
    @@logsDirPath = File.join(@@cachePath, 'log')

    attr_accessor :detail_git
    attr_accessor :detail_git_tag

    attr_accessor :detailObj
    attr_accessor :templates_git_url
    attr_accessor :codeSnippsets_git_url

    attr_accessor :templates_cache_path

    def initialize
      super
    end

    def self.cachePath
      @@cachePath
    end

    def self.configFilePath
      @@configFilePath
    end

    def self.configGitClonePath
      @@configGitClonePath
    end

    def self.logsDirPath
      @@logsDirPath
    end

    #从配置中心读取
    def self.loadConfig
      checkConfigFile?
      cc = DYAutomateConfig.fromFile(@@configFilePath)
      cc.loadDetailFromGit if cc
      cc
    end

    @private

    #检测config.rb是否存在
    def self.checkConfigFile?
      path = File.join(@@configFilePath)
      unless File.exist?(path)
        creatConfigFile
        puts "*** At first ,you should edit the config file at path<#{@@configFilePath}>!!"

        `open #{@@configFilePath}`
      end
    end

    #创建一个ConfigFile例子
    def self.creatConfigFile

      unless Dir.exist?(@@cachePath)
        Dir.mkdir(@@cachePath,0777)
      end

      File.new(@@configFilePath,'w+')

      f = File.open(@@configFilePath, "w+")
      demo = <<-EOF
  #DYAutomate 配置

  DYAutomateConfig.new do |c|
    # 配置的git地址
    c.detail_git = ""

    # 配置的git tag
    c.detail_git_tag = ""
  end
  EOF
      f.write(demo)
    end

    def templates_git_url
      url = ''
      url = @detailObj.templates_git_url if @detailObj
      url
    end

    def codeSnippsets_git_url
      url = ''
      url = @detailObj.codeSnippsets_git_url if @detailObj
      url
    end

    def templates_cache_path
      File.join(@@cachePath,'templates')
    end

    def check_git_dir
      unless Dir.exist?(@@configGitClonePath)
        Dir.mkdir(@@configGitClonePath,0777)
      end
    end

    def check_git_version
      versionFile = File.join(@@configGitClonePath,"version")
      isSame = false
      if File.exist?(versionFile)
        versionContent = File.open(versionFile, 'r:utf-8', &:read)
        if versionContent
          isSame = versionContent.chomp.eql?(@detail_git_tag.chomp)
        end
      end
      isSame
    end

    def clone_git
      check_git_dir
      if @detail_git_tag && @detail_git
        url = "#{@detail_git} --branch #{@detail_git_tag}"
        # url = "#{@git} "
        cloneOk = system "git clone #{url} #{@@configGitClonePath}"
        if cloneOk
          puts "clone #{url}成功。。。"
        else
          puts "clone #{url}失败！！"
        end
      else
        puts
      end
    end

    def loadDetailFromGit
      puts "加载配置。。。"
      check_git_dir
      unless check_git_version
        system("rm -rf #{@@configGitClonePath}")
        clone_git
      end
      path = File.join(@@configGitClonePath,'detailConfig.rb')
      @detailObj = DYAutomateConfigDetail.fromFile(path) if File.exist?(path)
      puts "加载配置成功。。。" if @detailObj
    end

  end
end